package;

import luxe.Color;
import phoenix.Vector;
import phoenix.Rectangle;
import luxe.importers.tiled.TiledObjectGroup;
import Action;
import luxe.Sprite;


class Building extends luxe.Entity {
    static public var buildings = new Array<Building>();
    static public var nodes = new Array<Vector>();
    static public var spawn_points = new Array<Vector>();
    public var sprite:Sprite;
    public var rect:luxe.Rectangle;
    public var spot:luxe.Rectangle;
    public var action:Null<ActionType>;

    public function new(name) {
        super();
        this.name = name;

        Building.buildings.push(this);
    }

    public function set_rect(obj:TiledObject) {
        rect = new luxe.Rectangle(obj.pos.x, obj.pos.y, obj.width, obj.height);
    }

    public function set_door(obj:TiledObject) {
        spot = new luxe.Rectangle(obj.pos.x, obj.pos.y, obj.width, obj.height);
    }

    override function onmouseup(e) {
        var pos:Vector = e.pos;
        if (rect == null) {
            return;
        }
        if (rect.point_inside(pos)) {
            sprite.color = new Color(1, 1, 1, 1);
            clicked();
        }
    }

    override function onmousemove(e) {
        var pos:Vector = e.pos;
        if (sprite.point_inside(pos)) {
            sprite.color = new Color(1.1,1.1,1.1,1);
        } else {
            sprite.color = new Color(1, 1, 1, 1);
        }
    }

    override function onmousedown(e) {
        var pos:Vector = e.pos;
        if (sprite.point_inside(pos)) {
            sprite.color = new Color(0.8, 0.8, 0.8,1);
        }
    }

    public static function debug() {
        for(n in Building.nodes) {
            new luxe.Sprite({
                size: new phoenix.Vector(10, 10),
                pos: n,
                color: new luxe.Color(0, 255, 0)
            });
        }

        for(b in Building.buildings) {
            new luxe.Sprite({
                size: new phoenix.Vector(10, 10),
                pos: new phoenix.Vector(b.spot.x, b.spot.y),
                color: new luxe.Color(0, 0, 1)
            });
        }
    }
    // /!\ Warning : Dangerous code /!\
    public static function find_path(a:Vector, b:Vector) {
        var start:Vector = new Vector(a.x, a.y);
        var end:Vector = new Vector(b.x, b.y);
        var selectedNodes = new Array<Vector>();
        var actualPosition = start;
        selectedNodes.push(actualPosition);

        //Sortir du batîments
        var nestedNode = new Vector();
        var minNode: Vector = new Vector();
        var minDistance:Float = 999999;
        for(n in Building.nodes) {
            var subVector:Vector = phoenix.Vector.Subtract(n, actualPosition);
            var distance:Float = subVector.length;
            if (distance < minDistance) {
                minDistance = distance;
                minNode = n;
            }
        }
        actualPosition = minNode;
        selectedNodes.push(actualPosition);

        // Aller au nouveau
        for (i in  0...10) {
            var minNode: Vector = new Vector();
            var minDistance:Float = 999999;
            for(n in Building.nodes) {
                var subVector:Vector = phoenix.Vector.Subtract(n, end);
                var distance:Float = subVector.length;
                if (
                        distance < minDistance && (
                            (n.x >= actualPosition.x-50 && n.x < actualPosition.x+50) ||
                            (n.y >= actualPosition.y-50 && n.y < actualPosition.y+50)
                        )
                    ) {
                    minDistance = distance;
                    minNode = n;
                }
            }

            if (minNode != actualPosition) {
                minNode.x += Reg.rand.int(-30, 30);
                minNode.y += Reg.rand.int(-30, 30);

                actualPosition = minNode;
                selectedNodes.push(actualPosition);
            }
        }
        selectedNodes.push(new Vector(end.x + Reg.rand.int(-30, 30), end.y + Reg.rand.int(-30, 30)));
        return selectedNodes;
    }

    public function clicked() {
        trace("You have clicked on : " + name);
        Reg.precheur.select_building(this);
    }

    public static function get_spawn_spot() {
        var random_spawn = Building.spawn_points[Reg.rand.int(Building.spawn_points.length)];
        return new Rectangle(random_spawn.x, random_spawn.y, 1, 1);
    }

    public static function get_building_name(name:String) {
        return Lambda.find(Building.buildings, function(x){return (name.toLowerCase() == x.name.toLowerCase());});
    }

    public static function add_node(node:Vector, spawn:Bool) {
        Building.nodes.push(node);
        if (spawn) {
            Building.spawn_points.push(node);
        }
    }
}
