package;

import luxe.tween.actuators.MotionPathActuator;
import luxe.tween.MotionPath;
import luxe.tween.actuators.GenericActuator;
import phoenix.Vector;
import phoenix.Color;
import Clock;


class LifeComponent extends luxe.Component {
    var planning:Map<ClockTime, Building>;
    public var place:Null<Building>;
    var motion:MotionPathActuator;
    public var actualPath:Array<Vector>;
    public var isMoving:Bool;

    public function new() {
        super({name: "life"});
    }

    override public function init() {
        this.reset_day();
        this.planning = generate_planning();
    }

    function generate_planning() {
        var planning = new Map<ClockTime, Building>();

        for (time in [MORNING, NOON, EVENING]) {
            var building = Building.buildings[Reg.rand.int(Building.buildings.length)];
            planning[time] = building;
        }

        return planning;
    }

    public function reset_day() {
        place = null;
    }

    public static function compute_distance(path:Array<Vector>) {
        var here = path[0];
        var dist = 0.;
        for (i in 1...path.length) {
            var vec = path[i];
            dist += vec.subtract(here).length;
            here = vec;
        }
        return dist;
    }

    public function clock(hour:ClockTime) {
        var spot;
        if (motion != null) {
            motion.stop(null, true, true);
        }
        if (hour == BEFORE) {
            var random_spot = Building.get_spawn_spot();
            this.entity.pos.set(random_spot.x, random_spot.y, 0, 0);
            return;
        } else if (hour == AFTER) {
            return;
        } else {
            place = planning[hour];
            spot = place.spot;
        }
        isMoving = true;

        var motionpath = new MotionPath();

        var path = Building.find_path(this.pos, new Vector(spot.x, spot.y));
        actualPath = path;
        for (point in path) {
            motionpath.line(point.x, point.y);
            // motionpath.bezier(point.x, point.y, point.x + Reg.rand.int(-100, 100), point.y + Reg.rand.int(-100, 100));
        }

        var distance = LifeComponent.compute_distance(path);
        var speed = Reg.rand.int(150, 400);
        var timing = distance/speed;

        Luxe.timer.schedule(Reg.rand.float(0, 2), function() {
            motion = new MotionPathActuator(this.entity.pos, timing, {x: motionpath.x, y: motionpath.y});
            motion.move();
            motion.onComplete(complete_motion);
        }, false);
    }

    override public function update(dt:Float) {
        // if (actualPath == null || !isMoving){
        //     return;
        // }
        // for(i in 1...actualPath.length) {
        //     Luxe.draw.line({
        //         immediate:true,
        //         p0 : actualPath[i-1],
        //         p1 : actualPath[i],
        //         color : new Color(0.5,0.2,0.2,1)
        //     });
        // }
    }

    function complete_motion() {
        var citizen:Citizen = cast this.entity;
        Reg.precheur.apply_action(citizen);
        isMoving = false;
    }
}
