package;

import luxe.tween.actuators.MotionPathActuator;
import luxe.tween.MotionPath;
import luxe.Entity;
import Clock;
import Action;
import luxe.Sprite;
import phoenix.Vector;
import luxe.Color;


class Precheur extends Entity {
    public var place:Null<Building>;
    public var actions:Map<ActionType, Sprite> = new Map<ActionType, Sprite>();
    public var original_positions = [
        AFFICHE => {x: 50, y: 50},
        OYEOYE => {x: 145, y: 50},
        PARLER => {x: 240, y: 50}
    ];
    // public var action:Null<Action>;
    public var selected_building:Null<Building>;
    public var actual_building:Null<Building>;
    public var selected_action:Null<ActionType>;
    public var actual_action:Null<ActionType>;
    public var motion:MotionPathActuator;

    public function new() {
        super();
        draw_action();
    }

    public function move(new_building:Building) {
        place = new_building;
    }

    public function action_time(time:ClockTime) {
        if (actual_action != null) {
            reset_pos(actual_action);
        }
        selected_building = null;
        selected_action = null;
        actual_action = null;
        actual_building = null;
    }

    public function reset_pos(action:ActionType) {
        actions[action].pos.x = original_positions[action].x;
        actions[action].pos.y = original_positions[action].y;
    }

    public function select_building(building:Building) {
        if (selected_action == null) {
            return;
        }
        if (selected_building != null) {
            return;
        }
        selected_building = building;
        selected_building.action = selected_action;
        end_of_actiontime();
    }

    public function end_of_actiontime() {
        actual_building = selected_building;
        actual_action = selected_action;
        Reg.clock.normal_speed();

        if (actual_building != null && actual_action != null) {
            var sprite = actions[actual_action];
            sprite.pos.x = actual_building.spot.x;
            sprite.pos.y = actual_building.spot.y;
            // var motionpath = new MotionPath();
            // motionpath.line(actual_building.spot.x, actual_building.spot.y);
            // trace(motionpath);
            // motion = new MotionPathActuator(sprite.pos, 2, {x: motionpath.x, y: motionpath.y});
            // motion.move();
        }
    }

    public function apply_action(citizen:Citizen) {
        var feature:CitizenFeature = cast citizen.get("citizen_feature");
        var life:LifeComponent = cast citizen.get("life");

        if (life.place != actual_building) {
            return;
        }

        if (actual_action == PARLER) {
            if (feature.is_weak) {
                feature.increase_conversion(Reg.rand.float(1, 4));
            }
        } else if (actual_action == AFFICHE) {
            feature.decrease_strength(Reg.rand.float(0.1, 0.3));
        } else if (actual_action == OYEOYE) {
            feature.increase_conversion(Reg.rand.float(0.5, 0.7));
        }
    }

    public function draw_action()  {
        trace("Draw action");
        var parlerTex = Luxe.resources.texture('assets/icones/parler.png');
        var afficheTex = Luxe.resources.texture('assets/icones/affiche.png');
        var oyeTex = Luxe.resources.texture('assets/icones/oye.png');
        for(prop in Type.allEnums(ActionType)) {
            var pos = original_positions[prop];
            if (prop == PARLER) {
                var parler: Sprite = new Sprite({
                    name:"parler",
                    texture: parlerTex,
                    color: new Color(0.9, 0.9, 0.9, 1),
                    size: new Vector(64, 64),
                    pos: new Vector(pos.x, pos.y),
                });
                actions[PARLER] = parler;
            } else if( prop == OYEOYE) {
                var oye: Sprite = new Sprite({
                    name:"oyeoye",
                    texture: oyeTex,
                    color: new Color(0.9, 0.9, 0.9, 1),
                    size: new Vector(64, 64),
                    pos: new Vector(pos.x, pos.y),
                });
                actions[OYEOYE] = oye;
            } else if( prop == AFFICHE) {
                var affiche: Sprite = new Sprite({
                    name:"affiche",
                    texture: afficheTex,
                    color: new Color(0.9, 0.9, 0.9, 1),
                    size: new Vector(64, 64),
                    pos: new Vector(pos.x, pos.y),
                });
                actions[AFFICHE] = affiche;
            }
        }
    }

    override function onmouseup(e) {
        var pos:Vector = e.pos;
        for(sprite in actions) {
            if (sprite.point_inside(pos)) {
                sprite.color = new Color(0.9, 0.9, 0.9, 1);
                clicked(sprite);
            }
        }
    }

    override function onmousemove(e) {
        var pos:Vector = e.pos;
        for(sprite in actions) {
            if (sprite.point_inside(pos)) {
                sprite.color = new Color(1,1,1,1);
            } else {
                sprite.color = new Color(0.9, 0.9, 0.9, 1);
            }
        }
    }

    override function onmousedown(e) {
        var pos:Vector = e.pos;
        for(sprite in actions) {
            if (sprite.point_inside(pos)) {
                sprite.color = new Color(0.8, 0.8, 0.8,1);
            }
        }
    }

    public function clicked(action:Sprite) {
        if (actual_action != null) {
            return;
        }
        if (! Reg.clock.time_to_action) {
            return;
        }
        if (action.name == "affiche") {
            selected_action = AFFICHE;
        } else if (action.name == "oyeoye") {
            selected_action = OYEOYE;
        } else if (action.name == "parler") {
            selected_action = PARLER;
        }
    }
}
