package map;

import luxe.Component;
import luxe.importers.tiled.TiledMap;
import phoenix.Texture;
import luxe.Vector;
import luxe.Color;

import luxe.collision.shapes.Polygon;
import luxe.collision.data.ShapeCollision;
import luxe.importers.tiled.TiledObjectGroup;


class MapLoader extends Component {
    var map:TiledMap;
    override function new(options:Dynamic, _map:String) {
        LoadMap(_map);
        super(options);
    }

    public function LoadMap(name:String) {
        map = new TiledMap({
            tiled_file_data: Luxe.resources.text(name).asset.text,
            asset_path: "assets/maps/",
        });

        map.display({ filter: FilterType.nearest });
        object_factory();
        load_image_layer();
    }

    public function load_image_layer() {
        for(layer in map.tiledmap_data.image_layers) {
            var building = Building.get_building_name(layer.name);
            var a = new luxe.Sprite({
                name:'image_layer.${layer.name}',
                centered: false, depth:-1,
                pos: new Vector(map.pos.x + layer.x, map.pos.y + layer.y),
                texture: Luxe.resources.texture('assets/maps/'+layer.image.source),
                color: new Color(1,1,1, layer.opacity),
                visible: layer.visible
            });

            if(building != null) {
                building.sprite = a;
            }
        } //each image_layer
    }

    public function object_factory() {
        var doors = new Map<String, TiledObject>();
        for(_group in map.tiledmap_data.object_groups) {
            for(_object in _group.objects) {
                switch(_object.type) {
                    case 'building': {
                        var building = new Building(_object.name);
                        building.set_rect(_object);
                    }
                    case 'door': {
                        doors[_object.name] = _object;
                    }
                    case 'node': {
                        var node = _object.pos;
                        var spawn = false;

                        if (_object.name == "spawn") {
                            spawn = true;
                        }
                        Building.add_node(node, spawn);
                    }
                } //switch type
            } //each object
        }

        for (building in Building.buildings) {
            var door = doors[building.name];
            building.set_door(door);
        }

        // Building.debug();
    }

    override function update(dt:Float) {
        //called every frame for you
    }

    override function onreset() {
        //called when the scene starts or restarts
    }
}
