package states;

import luxe.States;
import luxe.Scene;
import luxe.Color;
import luxe.Vector;
import luxe.importers.tiled.TiledMap;
import phoenix.Texture;
import luxe.Text;
import luxe.Entity;

import luxe.Input;
import map.MapLoader;
import Clock;
import luxe.Sprite;
import luxe.tween.Actuate;
import luxe.tween.easing.Quad;


class Play extends State {
    var playScene:Scene;
    var help:Sprite;
    var isShow:Bool;
    var win:Sprite;

    public function new() {
        super({ name: 'Play' });
        playScene = new Scene('Play');
        help = new Sprite({
            name : "clock",
            pos : Luxe.screen.mid,
            texture : Luxe.resources.texture("assets/help.png"),
            color : new Color(1,1,1,0),
            depth : 1000,
            scene : playScene
        });

        isShow = false;

        win = new Sprite({
            name: "win",
            pos: Luxe.screen.mid,
            texture: Luxe.resources.texture("assets/victory.png"),
            color : new Color(1,1,1,0),
            depth : 1000
        });
    }

    override function onenter<T>(_:T) {
        trace("Load Test");
        var map = new Entity({name:"map", scene: playScene});
        map.add(new MapLoader({name:"map_loader"},"assets/maps/level0.tmx"));

        new Citizen("Bob");
        new Citizen("Patrick");
        new Citizen("Sandy");
        new Citizen("Carlo");
        new Citizen("Eugene");
        new Citizen("Plankton");
        new Citizen("Garry");

        Reg.precheur = new Precheur();
        Reg.clock = new Clock(7.5, 23.5, 1200);
    }

    public function show_help() {
        Actuate.tween(help.color,0.5,{a:1}).ease (Quad.easeIn);
    }

    public function close_help() {
        Actuate.tween(help.color,0.5,{a:0}).ease (Quad.easeOut);
    }

    public function toggle_help() {
        if(isShow) {
            close_help();
        } else {
            show_help();
        }
        isShow = !isShow;
    }

    public function show_win(day:Int) {
        Actuate.tween(win.color, 1, {a: 1}).ease (Quad.easeIn);

        var text = Luxe.draw.text({
            align : TextAlign.center,
            color : new Color(1, 1, 1, 0),
            pos : new Vector(Luxe.screen.mid.x, Luxe.screen.mid.y-40),
            point_size : 60,
            text : "" + day,
            depth: 1001
        });

        Actuate.tween(text.color, 1, {a: 1});
    }

    override function onkeyup( e:KeyEvent ) {
        //quitter l'appli
        if(e.keycode == Key.key_h) {
            toggle_help();
        }
    } //onkeyup

    override function onmouseup(e) {
        var pos:Vector = e.pos;
        if (help.point_inside(pos)) {
            close_help();
        }
    }

    override function onleave<T>(_:T) {
        playScene.reset();
        playScene.destroy();
        for(entity in playScene.entities) {
            playScene.remove(entity);
        }
    }
}
